@@ -1200,12 +1200,12 @@ prepare_blt_clear(struct sna *sna,
 static bool
 prepare_blt_fill(struct sna *sna,
 		 struct sna_composite_op *op,
-		 PicturePtr source)
+		 uint32_t pixel)
 {
 	DBG(("%s\n", __FUNCTION__));
 
 	if (op->dst.bo == NULL) {
-		op->u.blt.pixel = get_solid_color(source, op->dst.format);
+		op->u.blt.pixel = pixel;
 		op->blt = blt_composite_fill__cpu;
 		op->box   = blt_composite_fill_box__cpu;
 		op->boxes = blt_composite_fill_boxes__cpu;
@@ -1228,8 +1228,7 @@ prepare_blt_fill(struct sna *sna,
 
 	if (!sna_blt_fill_init(sna, &op->u.blt, op->dst.bo,
 			       op->dst.pixmap->drawable.bitsPerPixel,
-			       GXcopy,
-			       get_solid_color(source, op->dst.format)))
+			       GXcopy, pixel))
 		return false;
 
 	return begin_blt(sna, op);
@@ -1975,6 +1974,13 @@ static bool source_is_gpu(PixmapPtr pixmap, const BoxRec *box)
 				      PICT_FORMAT_G(format),		\
 				      PICT_FORMAT_B(format))
 
+static bool
+is_clear(PixmapPtr pixmap)
+{
+	struct sna_pixmap *priv = sna_pixmap(pixmap);
+	return priv && priv->clear;
+}
+
 bool
 sna_blt_composite(struct sna *sna,
 		  uint32_t op,
@@ -1992,6 +1998,7 @@ sna_blt_composite(struct sna *sna,
 	int16_t tx, ty;
 	BoxRec dst_box, src_box;
 	uint32_t alpha_fixup;
+	uint32_t color, hint;
 	bool was_clear;
 	bool ret;
 
@@ -2024,25 +2031,30 @@ sna_blt_composite(struct sna *sna,
 	} else
 		sna_render_picture_extents(dst, &dst_box);
 
-	bo = sna_drawable_use_bo(dst->pDrawable, PREFER_GPU, &dst_box, &tmp->damage);
-	if (bo && !kgem_bo_can_blt(&sna->kgem, bo)) {
-		DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
-		     __FUNCTION__, bo->tiling, bo->pitch));
-		return false;
-	}
-
 	tmp->dst.format = dst->format;
 	tmp->dst.width = tmp->dst.pixmap->drawable.width;
 	tmp->dst.height = tmp->dst.pixmap->drawable.height;
 	get_drawable_deltas(dst->pDrawable, tmp->dst.pixmap,
 			    &tmp->dst.x, &tmp->dst.y);
-	tmp->dst.bo = bo;
 
 	if (op == PictOpClear) {
 clear:
 		if (was_clear)
 			return prepare_blt_nop(sna, tmp);
 
+		hint = 0;
+		if (can_render(sna))
+			hint |= PREFER_GPU;
+		if (dst->pCompositeClip->data == NULL)
+			hint |= IGNORE_CPU;
+		tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+						  &dst_box, &tmp->damage);
+		if (tmp->dst.bo && !kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+			DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
+			     __FUNCTION__, tmp->dst.bo->tiling, tmp->dst.bo->pitch));
+			return false;
+		}
+
 		if (!tmp->dst.bo) {
 			RegionRec region;
 
@@ -2067,11 +2079,26 @@ clear:
 			goto clear;
 
 		if (op != PictOpSrc) {
-			DBG(("%s: unsuported op [%d] for blitting\n",
+			DBG(("%s: unsupported op [%d] for blitting\n",
 			     __FUNCTION__, op));
 			return false;
 		}
 
+		color = get_solid_color(src, tmp->dst.format);
+fill:
+		hint = 0;
+		if (can_render(sna))
+			hint |= PREFER_GPU;
+		if (dst->pCompositeClip->data == NULL)
+			hint |= IGNORE_CPU;
+		tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+						  &dst_box, &tmp->damage);
+		if (tmp->dst.bo && !kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+			DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
+			     __FUNCTION__, tmp->dst.bo->tiling, tmp->dst.bo->pitch));
+			return false;
+		}
+
 		if (!tmp->dst.bo) {
 			RegionRec region;
 
@@ -2083,11 +2110,11 @@ clear:
 				return false;
 		}
 
-		return prepare_blt_fill(sna, tmp, src);
+		return prepare_blt_fill(sna, tmp, color);
 	}
 
 	if (!src->pDrawable) {
-		DBG(("%s: unsuported procedural source\n",
+		DBG(("%s: unsupported procedural source\n",
 		     __FUNCTION__));
 		return false;
 	}
@@ -2102,7 +2129,7 @@ clear:
 		op = PictOpSrc;
 
 	if (op != PictOpSrc) {
-		DBG(("%s: unsuported op [%d] for blitting\n",
+		DBG(("%s: unsupported op [%d] for blitting\n",
 		     __FUNCTION__, op));
 		return false;
 	}
@@ -2125,6 +2152,13 @@ clear:
 		goto clear;
 	}
 
+	src_pixmap = get_drawable_pixmap(src->pDrawable);
+	if (is_clear(src_pixmap)) {
+		color = color_convert(sna_pixmap(src_pixmap)->clear_color,
+				      src->format, tmp->dst.format);
+		goto fill;
+	}
+
 	alpha_fixup = 0;
 	if (!(dst->format == src_format ||
 	      dst->format == alphaless(src_format) ||
@@ -2158,7 +2192,6 @@ clear:
 			return false;
 	}
 
-	src_pixmap = get_drawable_pixmap(src->pDrawable);
 	get_drawable_deltas(src->pDrawable, src_pixmap, &tx, &ty);
 	x += tx + src->pDrawable->x;
 	y += ty + src->pDrawable->y;
@@ -2184,6 +2217,23 @@ clear:
 	src_box.x2 = x + width;
 	src_box.y2 = y + height;
 	bo = NULL;
+
+	hint = 0;
+	if (can_render(sna))
+		hint |= PREFER_GPU;
+	if (dst->pCompositeClip->data == NULL)
+		hint |= IGNORE_CPU;
+	if (source_is_gpu(src_pixmap, &src_box))
+		hint |= FORCE_GPU;
+
+	tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
+					  &dst_box, &tmp->damage);
+	if (tmp->dst.bo && !kgem_bo_can_blt(&sna->kgem, tmp->dst.bo)) {
+		DBG(("%s: can not blit to dst, tiling? %d, pitch? %d\n",
+		     __FUNCTION__, tmp->dst.bo->tiling, tmp->dst.bo->pitch));
+		return false;
+	}
+
 	if (tmp->dst.bo || source_is_gpu(src_pixmap, &src_box))
 		bo = __sna_render_pixmap_bo(sna, src_pixmap, &src_box, true);
 	if (bo) {
@@ -2229,7 +2279,7 @@ put:
 			region.data = NULL;
 
 			if (!sna_drawable_move_region_to_cpu(dst->pDrawable, &region,
-							MOVE_INPLACE_HINT | MOVE_READ | MOVE_WRITE))
+							     MOVE_INPLACE_HINT | MOVE_READ | MOVE_WRITE))
 				return false;
 		}
 
@@ -2294,7 +2344,7 @@ sna_blt_composite__convert(struct sna *sna,
 	if (op == PictOpOver && PICT_FORMAT_A(tmp->src.pict_format) == 0)
 		op = PictOpSrc;
 	if (op != PictOpSrc) {
-		DBG(("%s: unsuported op [%d] for blitting\n",
+		DBG(("%s: unsupported op [%d] for blitting\n",
 		     __FUNCTION__, op));
 		return false;
 	}
